// -*- C++ -*-
// -*- coding: utf-8 -*-
//
// michael a.g. aïvázis <michael.aivazis@para-sim.com>
// (c) 1998-2022 all rights reserved

// code guard
#if !defined(pyre_version_h)
#define pyre_version_h


// support
#include <tuple>
#include <string>


// put all version related symbols in their own namespace
// N.B.: version information is made available two different ways in order to enable detecting
// inconsistencies between the headers accessible at compile time and the library available
// at run time
namespace pyre::version {
    // type aliases
    using string_t = std::string;

    // my version is a tuple of three integers and the {git} hash
    using version_t = std::tuple<int, int, int, string_t>;

    // the version of the {pyre} library
    auto version() -> version_t;

    // the version visible through the headers
    // clang-format off
    const int major = 1;
    const int minor = 11;
    const int micro = 2;
    const string_t revision = "";
    // clang-format on

}    // namespace pyre::version


#endif

// end of file
